@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2022	PgPool Global Development Group
+ * Copyright (c) 2003-2023	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -226,7 +226,7 @@ typedef struct
 	char		*unix_socket_group;			/* owner group of pgpool sockets */
 	int			unix_socket_permissions;	/* pgpool sockets permissions */
 	char	   *wd_ipc_socket_dir;	/* watchdog command IPC socket directory */
-	char	   *pcp_socket_dir; /* PCP socket directory */
+	char	   **pcp_socket_dir; /* PCP socket directory */
 	int			num_init_children;	/* Maximum number of child to
 										 * accept connections */
 	int			min_spare_children;		/* Minimum number of idle children */
@@ -264,6 +264,7 @@ typedef struct
 	bool		replication_mode;	/* replication mode */
 	bool		log_connections;	/* logs incoming connections */
 	bool		log_disconnections;	/* logs closing connections */
+	bool		log_pcp_processes;	/* logs pcp processes */
 	bool		log_hostname;	/* resolve hostname */
 	bool		enable_pool_hba;	/* enables pool_hba.conf file
 									 * authentication */
@@ -398,6 +399,8 @@ typedef struct
 	bool		log_statement;	/* logs all SQL statements */
 	bool		log_per_node_statement; /* logs per node detailed SQL
 										 * statements */
+	bool		notice_per_node_statement; /* logs notice message for per node detailed SQL
+										 * statements */
 	bool		log_client_messages;	/* If true, logs any client messages */
 	char	   *lobj_lock_table;	/* table name to lock for rewriting
 									 * lo_creat */
@@ -414,6 +417,7 @@ typedef struct
 	int			num_listen_addresses;	/* number of entries in listen_addresses */
 	int			num_pcp_listen_addresses;	/* number of entries in pcp_listen_addresses */
 	int			num_unix_socket_directories;	/* number of entries in unix_socket_directories */
+	int			num_pcp_socket_directories;	/* number of entries in pcp_socket_dir */
 	int			num_read_only_function_list;	/* number of functions in
 											 * read_only_function_list */
 	int			num_write_function_list;	/* number of functions in
@@ -499,6 +503,17 @@ typedef struct
 	int			current_memqcache_table_pattern_size;	/* size of the regex
 														 * pattern array */
 
+	/*
+	 * user_redirect_preference_list =
+	 * 'postgres:primary,user[0-4]:1,user[5-9]:2'
+	 */
+	char	   *user_redirect_preference_list;	/* raw string in
+													 * pgpool.conf */
+	RegArray   *redirect_usernames;	/* Precompiled regex patterns for db
+									 * preference list */
+	Left_right_tokens *user_redirect_tokens;	/* db redirect for dbname and node
+											 * string */
+
 	/*
 	 * database_redirect_preference_list =
 	 * 'postgres:primary,mydb[0-4]:1,mydb[5-9]:2'
